<?php namespace Avans;

use Avans\parts\course;
use Avans\parts\gifts;
use Avans\parts\referer;
use stdClass;
use WC_Order;
use WP_User;

class actions
{
    public function __construct()
    {
        if (!is_admin()) {
            add_action('wp_footer', array($this,'avans_load_footer'));
        }
        add_action('transition_comment_status', array($this,'avans_status_transition'), 20, 3);
        add_action('publish_post', array($this,'avans_publish_post'), 10, 2);
        add_action('publish_product', array($this,'avans_publish_product'), 10, 2);
        add_action('woocommerce_thankyou', array($this,'avans_wc_thankyou'), 10, 1);
        $status = (null !== AVANS_SETTINGS['score_settings']['purchase']['status']) ? AVANS_SETTINGS['score_settings']['purchase']['status'] : '';
        if( empty($status) || in_array($status,array('after-purchase','after-order-compeleted')) ){
            add_action('woocommerce_order_status_completed', array($this,'avans_after_order_completed'), 10, 3);
        }
        add_action('woocommerce_order_status_cancelled', array($this,'avans_after_order_cancelled'), 10, 3);
        add_action('woocommerce_order_status_refunded', array($this,'avans_after_order_cancelled'), 10, 3);
        add_action('gform_after_submission', array($this,'avans_gravity_after_submission'), 10, 2);
        add_action('wp', array($this,'check_cron_jobs'));
        add_action('avans_daily_event', array($this,'avans_daily_schedule'));
        add_action('init', array($this,'apply_score_in_custom_transition_status'));
        add_action('init', array($this,'avans_set_coocki'));
        add_action( 'delete_user', array($this,'remove_user'),10,1 );

        new course;
    }


    public function avans_load_footer()
    {
        $this->avans_post_types_score();
        $this->avans_register_user();
        $this->avans_daily_visit();
        $this->avans_share_links();
        $this->avans_load_css_and_js();
    }

    public function remove_user($user_id){
        data::remove_user_actions($user_id);
    }

    function avans_load_css_and_js(){
        include_once AVANS_PATH.'templates/front/scripts/style.php';
        include_once AVANS_PATH.'templates/front/scripts/script.php';
    }

    public function avans_post_types_score()
    {
        if (is_singular() && is_user_logged_in()) {
            include_once AVANS_PATH.'includes/front/actions/post-types.php';
        }
    }

    public function apply_score_in_custom_transition_status(){
        if(function_exists('wc_get_order_statuses')){
            $statuses = wc_get_order_statuses();
            $unsetStatus = array('wc-processing','wc-completed','wc-pending', 'wc-on-hold' , 'wc-cancelled' , 'wc-refunded' , 'wc-failed');
            foreach($unsetStatus as $key) {
                unset($statuses[$key]);
            }
            foreach($statuses as $key=>$status){
                $stat = str_replace('wc-','',$key);
                add_action('woocommerce_order_status_'.$stat, array($this,'avans_after_order_transition'), 10, 3);
            }
        }
    }

 
    
    public static function avans_course_completed($user_id,$course_id){

        do_action('avans_before_custom_course_completed',$user_id, $course_id);

        if(AVANS_SETTINGS['score_settings']['course']['complete_course']['active'] !== true) return;

        $user_complete_courses = (array)get_user_meta($user_id,AVANS_PREFIX.'completed_courses',true);
        if(in_array($course_id,$user_complete_courses)) return;

        $details = array(
            array(
                'key' => 'course_id',
                'value' => $course_id
            )
        );
        $result = user::add_log((int)$user_id,'lms_course_complete',$details);

        if((int)$result > 0){
            array_push($user_complete_courses,$course_id);
            update_user_meta($user_id,AVANS_PREFIX.'completed_courses',$user_complete_courses);
        }

    }

    public static function avans_lesson_completed($user_id,$course_id,$lesson_id){
        if(AVANS_SETTINGS['score_settings']['course']['complete_lesson']['active'] !== true) return;

        $user_complete_lessons = (array)get_user_meta($user_id,AVANS_PREFIX.'completed_lessons',true);
        $obj = $course_id.'-'.$lesson_id;
        if(in_array($obj,$user_complete_lessons)) return;

        $details = array(
            array(
                'key' => 'course_id',
                'value' => $course_id
            ),
            array(
                'key' => 'lesson_id',
                'value' => $lesson_id
            )
        );
        $result = user::add_log((int)$user_id,'lms_lesson_complete',$details);

        if((int)$result > 0){
            array_push($user_complete_lessons,$obj);
            update_user_meta($user_id,AVANS_PREFIX.'completed_lessons',$user_complete_lessons);
        }

    }

    public static function avans_quiz_passed($user_id,$course_id,$quiz_id){
        if(AVANS_SETTINGS['score_settings']['course']['passed_test']['active'] !== true) return;

        $user_quizzes_passed = (array)get_user_meta($user_id,AVANS_PREFIX.'quizzes_passed',true);
        $obj = $course_id.'-'.$quiz_id;
        if(in_array($obj,$user_quizzes_passed)){
            return;
        }

        $details = array(
            array(
                'key' => 'course_id',
                'value' => $course_id
            ),
            array(
                'key' => 'quiz_id',
                'value' => $quiz_id
            )
        );
        $result = user::add_log((int)$user_id,'lms_quiz_passed',$details);

        if((int)$result > 0){
            array_push($user_quizzes_passed,$obj);
            update_user_meta($user_id,AVANS_PREFIX.'quizzes_passed',$user_quizzes_passed);
        }

    }

    public static function avans_user_enrolled_in_course($user_id, $course_id){

        do_action('avans_before_custom_course_enrolled',$user_id, $course_id);

        if(AVANS_SETTINGS['score_settings']['course']['enroll_course']['active'] !== true) return;

        $user_course_enrolled = (array)get_user_meta($user_id,AVANS_PREFIX.'course_enrolled',true);       
        if(in_array($course_id,$user_course_enrolled)) return;

        $details = array(
            array(
                'key' => 'course_id',
                'value' => $course_id
            ),
        );

        $result = user::add_log((int)$user_id,'lms_course_enrolled',$details);

        if((int)$result > 0){
            array_push($user_course_enrolled,$course_id);
            update_user_meta($user_id,AVANS_PREFIX.'course_enrolled',$user_course_enrolled);
        }

    }

    
    public function avans_daily_visit()
    {
        if (is_user_logged_in()) {
            $settings = AVANS_SETTINGS['score_settings']['daily'];
            if ($settings['active']) {
                include_once AVANS_PATH.'includes/front/actions/daily-visit.php';
            }
        }
    }


    // do register actions such as increase score or send coupon sms or check referer
    public function avans_register_user()
    {
        $user = wp_get_current_user();
        if(!is_user_logged_in()) return;
        $settings = AVANS_SETTINGS['score_settings']['register'];
        $ref_settings = AVANS_SETTINGS['score_settings']['ref-register'];
        $is_first_login = empty(get_user_meta($user->ID, AVANS_PREFIX.'first_login', true)) ? true : false; 
        $register_actions_completed = (array)get_user_meta($user->ID, AVANS_PREFIX.'actions_after_register', true);
        
        if ( $is_first_login  ) { 
            $score = 0;

            //add register score
            if($settings['active'] && !in_array('score',$register_actions_completed) ) {
                $score = $this->apply_register_score($user);
                array_push($register_actions_completed,'score');
                update_user_meta($user->ID, AVANS_PREFIX.'actions_after_register',$register_actions_completed);
            }
            
            //send wc-coupon for user mobile where user mobile not empty
            if(AVANS_SETTINGS['coupen_settings']['after-register']['active'] === 'true' && !in_array('coupon',$register_actions_completed) ){
                actions::avans_send_coupon('after-register', $user->ID);
                array_push($register_actions_completed,'coupon');
                update_user_meta($user->ID, AVANS_PREFIX.'actions_after_register',$register_actions_completed);
            }

            //check referer 
            if($ref_settings['active'] === true && !in_array('ref-check',$register_actions_completed) ){
                $this->apply_register_referer_score($user);
                array_push($register_actions_completed,'ref-check');
                update_user_meta($user->ID, AVANS_PREFIX.'actions_after_register',$register_actions_completed);
            }
            
            update_user_meta($user->ID, AVANS_PREFIX.'first_login', 1);
            do_action('avans_after_register_user',$user,$score);
        }
    }

    public function apply_register_score($user){
        $option = array();

        $score = user::add_log($user->ID, 'register');

        if ((int)$score > 0) {
            $option['score'] = $score;
            $unit = func::get_score_label();
            $option['text'] = 'شما بابت ثبت نام در سایت  '.$option['score']." $unit گرفتید";
            if (!empty($settings['text'])) {
                $option['text']  = str_replace("{score}", $option['score'], $settings['text']);
            }
            
            $data = array('text' => $option['text']);
            wp_localize_script("avans_score_register_script", "avs_show_alert", $data);
            wp_enqueue_script("avans_score_register_script");
        }

        return $score;
    }

    public function apply_register_referer_score($user){

        $ref_code = get_user_meta($user->ID, AVANS_PREFIX.'register_refID', true);
        if (func::is_set($_COOKIE['avans-referal-id'])) {
            $ref_code = $_COOKIE['avans-referal-id'];
        }

        if (empty($ref_code)) return;

        $referers = get_users(array('meta_key' =>  AVANS_PREFIX.'user_ref_code' , 'meta_value' => $ref_code));
        $current_user_ref_code = get_user_meta($user->ID, AVANS_PREFIX.'user_ref_code', true);

        if (count($referers) < 1  || $ref_code == $current_user_ref_code) return;

        $referer = $referers[0];

        if (!user::is_permision_get_score('ref-register', $referer->ID)) return;

        $details = array(
            array(
                'key' => 'user-id',
                'value' => $user->ID
            )
        );

        user::add_log($referer->ID, 'ref-register', $details);
        $usersRefs = (array)get_user_meta($referer->ID, AVANS_PREFIX.'users_refs', true);
        array_push($usersRefs, $user->ID);
        update_user_meta($referer->ID, AVANS_PREFIX.'users_refs', $usersRefs);
        update_user_meta($user->ID, AVANS_PREFIX.'register_user_ref', $referer->ID);
        do_action('avans_after_referer_code_used_for_register', $referer, $user, $usersRefs);
        

        if (!isset($_COOKIE['avans-referal-id'])) {
            $timeset = (int)AVANS_SETTINGS['referer_settings']['referer']['day_coocki'] * 86400;
            setcookie('avans-referal-id', $ref_code, (time()+$timeset), "/");
        }


    }

    public function avans_status_transition($new_status, $old_status, $comment)
    {
        $settings = AVANS_SETTINGS['score_settings']['comment'];

        if ($comment->user_id > 0) {

            if ($old_status != $new_status) {
                if (!user::is_permision_get_score('comment', $comment->user_id)) {
                    return;
                }

                if($new_status == 'approved' && $settings['once_comment']){
                    if(data::post_is_commented($comment)){
                        return;
                    }
                }

                $has_bought = false;

                if (class_exists('WooCommerce') && get_post_type($comment->comment_post_ID) == 'product') {
                    $has_bought = wc_customer_bought_product(null, $comment->user_id, $comment->comment_post_ID);
                }

                $status = get_comment_meta($comment->comment_ID, AVANS_PREFIX.'score_status', true);

                if ($new_status == 'approved' && empty($status)) {

                    if($settings['active']){

                        if ($has_bought) {
                            $score = !empty($settings['score_wc']) ? (int)$settings['score_wc'] : 1;
                        } else {
                            $score = !empty($settings['score']) ? (int)$settings['score'] : 1;
                        }

                        $score = data::get_final_action_score('comment', $comment->user_id, $score);
                        if ($score > 0) {
                            update_comment_meta($comment->comment_ID, AVANS_PREFIX.'score_status', true);
                        }

                        $details = [
                            array(
                                'key' => 'post_id',
                                'value' => $comment->comment_post_ID
                            ),
                        ];

                        if (is_int($score)) {
                            user::add_log($comment->user_id, 'comment', $details , $score);
                        }
                    
                    }
                    
                    if( $has_bought && !data::post_is_commented($comment) ){
                        self::avans_send_coupon('after-comment-product', $comment->user_id);
                    }
                    
                    do_action('avans_after_comment', $comment);

                }
 
            }

            
        }
    }

    public function avans_publish_post($ID, $post)
    {
        $settings = AVANS_SETTINGS['score_settings']['publish'];
        $published = (array)get_user_meta($post->post_author, AVANS_PREFIX.'posts_published', true);
        if (!in_array($ID, $published) && $settings['active']) {
            include_once AVANS_PATH.'includes/front/actions/publish-post.php';
        }
    }

        public function avans_publish_product($ID, $post)
    {
        $settings = AVANS_SETTINGS['score_settings']['publish-product'];
        $published = (array)get_user_meta($post->post_author, AVANS_PREFIX.'product_published', true);

        if (!in_array($ID, $published) && $settings['active']) {
            include_once AVANS_PATH.'includes/front/actions/publish-product.php';
        }

    }

    public function avans_after_order_transition($order_id){
        $status = str_replace('woocommerce_order_status_','',current_filter());
        $settingStatus = (null !== AVANS_SETTINGS['score_settings']['purchase']['status']) ? AVANS_SETTINGS['score_settings']['purchase']['status'] : '';
        $settingStatus = str_replace('wc-','',$settingStatus);
        if($settingStatus == $status){
            $this->avans_after_order_completed($order_id);
        }
    }


    public function avans_wc_thankyou($order_id)
    {
    
        $order_viewed = get_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);
        $is_sub_order = ( (int)wp_get_post_parent_id( $order_id ) > 0 ) ? true : false;

        $settings = AVANS_SETTINGS['score_settings']['purchase'];
        $referer = AVANS_SETTINGS['referer_settings']['referer'];
        
        $order = wc_get_order($order_id);  
        $user = $order->get_user();
        
        if(!$order || $is_sub_order){
            return;
        }

        if ($settings['status'] == 'after-purchase' && $order->has_status(array('processing' , 'completed')) ) {
            if (empty($order_viewed) && $user instanceof WP_User) {

                $this->avans_apply_wc_order_score($order);

                if ($referer['active']) {
                    $code_referal = WC()->session->get(AVANS_PREFIX.'referal_code');
                    if (func::is_set($code_referal)) {
                        referer::apply_referer_commission($order, $user->ID, $code_referal);
                        update_user_meta($user->ID, AVANS_PREFIX.'wc_ref_code', $code_referal);
                        update_post_meta($order->ID, AVANS_PREFIX.'wc_ref_order', $code_referal);
                        WC()->session->__unset(AVANS_PREFIX.'referal_code');
                    }
                }

                $this->avans_apply_wc_order_coupon($order);

                update_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);
                do_action('avans_after_purchase', $order, $user);
            }
            if(empty($order_viewed) && !$user instanceof WP_User){
                $this->avans_apply_wc_order_coupon($order);
                update_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);
            }
        } else {
            if ($referer['active'] && $user) {
                $code_referal = WC()->session->get(AVANS_PREFIX.'referal_code');
                if (func::is_set($code_referal)) {
                    update_user_meta($user->ID, AVANS_PREFIX.'wc_ref_code', $code_referal);
                    update_post_meta($order->ID, AVANS_PREFIX.'wc_ref_order', $code_referal);
                    WC()->session->__unset(AVANS_PREFIX.'referal_code');
                }
            }
        }


        
    }

    public function avans_apply_wc_order_score($order){
        $settings = AVANS_SETTINGS['score_settings']['purchase'];
        $price = data::get_total_order($order);

        $sum_score = 0;
        $min_price = !empty($settings['min_price']) ? (int)$settings['min_price'] : 0 ;
        $user = $order->get_user();

        if ($settings['active'] && $order->get_payment_method() != 'wallet' && $price >= $min_price) {
            $sum_score = data::get_sum_score_for_purchase($user->ID, $order);
            $is_permision = user::is_permision_get_score('wc_order', $user->ID);
            $sum_score =  user::get_valid_score($user->ID, $sum_score, true);

            if ($sum_score > 0 &&  $is_permision) {
                $details = [
                    array(
                        'key' => 'order_id',
                        'value' => $order->ID,
                    )
                ];
                update_post_meta($order->ID,AVANS_PREFIX.'order_score',$sum_score);
                user::add_log($user->ID, 'wc_order', $details, $sum_score);
            }


            if ($sum_score > 0  && $settings['status'] == 'after-purchase' && $is_permision) {
                $option = [];
                $option['text'] = 'شما بابت خرید از ما '.$sum_score.' '.func::get_score_label().' '.'گرفتید';
                if (!empty($settings['text'])) {
                    $option['text']  = str_replace("{score}", $sum_score, $settings['text']);
                }


                $data = array(
                    'text' => $option['text'],
                );
                
                wp_localize_script("avans_score_order_localize_script", "avs_show_alert", $data);
                wp_enqueue_script("avans_score_order_localize_script");
            }
        }
    }

    public function avans_apply_wc_order_coupon($order){
        $coupon = AVANS_SETTINGS['coupen_settings']['after-order'];

        if ($coupon['active'] !== 'true') {
            return;
        }

        $user = $order->get_user();
        $price = data::get_total_order($order);
        $phone = $order->get_billing_phone();
        $min_price = (int)$coupon['min_price'];


        if ($user instanceof WP_User) {

            $total_orders = user::get_total_orders_price($user->ID);
            
            $min_orders = (int)$coupon['min_orders'];

            if ($coupon['repeat'] =='always'):

                $limit_time_value = (int)$coupon['limit_time_value'];
                $limit_time_unit = $coupon['limit_time_unit'];
                $permison = true;

                $last_time_send = (int)get_user_meta($user->ID, AVANS_PREFIX.'last_use_coupen_time',true);
                if((int)$last_time_send > 0  && (int)$limit_time_value > 0 ){
                    $permison = data::is_permision_send_coupon_by_time($limit_time_value,$limit_time_unit,$last_time_send);
                }
                if ($permison) {
                    update_user_meta($user->ID, AVANS_PREFIX.'is_user_use_coupen', '');
                }

            endif;

            $use_coupen = get_user_meta($user->ID, AVANS_PREFIX.'is_user_use_coupen', true);

            if ($price >= $min_price && $total_orders >= $min_orders && empty($use_coupen) && func::is_set($phone)) {
                actions::avans_send_coupon('after-order', $user->ID, $order);
                update_user_meta($user->ID, AVANS_PREFIX.'is_user_use_coupen', true);
                update_user_meta($user->ID, AVANS_PREFIX.'last_use_coupen_time', time());
            }
        }


        if(!$user instanceof WP_User && $coupon['guest_user'] == 'true'){
            if ( $price >= $min_price && !empty($phone) ){
                actions::avans_send_coupon('after-order', null, $order);
            }
        }

    }

    public function avans_after_order_completed($order_id)
    {
        $is_sub_order = ( (int)wp_get_post_parent_id( $order_id ) > 0 ) ? true : false;

        if($is_sub_order){
            return;
        }

        $order_viewed = get_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);
        $referer = AVANS_SETTINGS['referer_settings']['referer'];
        $order = wc_get_order($order_id);
        $user = $order->get_user();

        $is_prize = get_post_meta($order_id, AVANS_PREFIX.'is_order_prize', true);
        if ($is_prize == 'true') {
            return;
        }


        if (empty($order_viewed) && $user instanceof WP_User) {

            $this->avans_apply_wc_order_score($order);

            if ($referer['active']) {
                // $user_ref = get_user_meta($user->ID, AVANS_PREFIX.'wc_ref_code', true);
                $order_ref = get_post_meta($order_id, AVANS_PREFIX.'wc_ref_order', true);

                if (!empty($order_ref)) {
                    referer::apply_referer_commission($order,$user->ID,$order_ref);
                }
            }

            $this->avans_apply_wc_order_coupon($order);

            update_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);

            do_action('avans_after_purchase', $order, $user);
        }
        
        if(empty($order_viewed) && !$user instanceof WP_User){
            $this->avans_apply_wc_order_coupon($order);
            update_post_meta($order_id, AVANS_PREFIX.'order_viewed', true);
        }
        
    }

    public function avans_after_order_cancelled($order_id){
        $order_score = (int)get_post_meta($order_id, AVANS_PREFIX.'order_score', true);
        if($order_score > 0){
            $order = wc_get_order($order_id);
            $user = $order->get_user();

            if($user->ID == 0){
                return;
            }

            $details = [['key' => 'order_id','value' => $order_id]];
            user::add_log((int)$user->ID,'wc_order_cancelled',$details,-$order_score);
        }
        delete_post_meta($order_id, AVANS_PREFIX.'order_viewed');
    }

    public function avans_set_coocki()
    {
        if ( isset($_GET['referer']) && !empty($_GET['referer']) ) {
            $nameCookie = 'avans-referal-id';
            $users = get_users(array('meta_key' => AVANS_PREFIX.'user_ref_code', 'meta_value' => $_GET['referer']));
            if (count($users) != 0 && !isset($_COOKIE[$nameCookie])) {
                $timeset = (int)AVANS_SETTINGS['referer_settings']['referer']['day_coocki'];
                $timeset = (func::is_set($timeset) &&  $timeset > 0) ? ($timeset * 86400) : 86400;
                setcookie($nameCookie, $_GET['referer'], (time()+$timeset), "/");
            }
        }
    }

    public function avans_share_links()
    {
        $settings = (array)AVANS_SETTINGS['score_settings']['share'];
        if ($settings['active'] && isset($_GET['referer']) && !empty($_GET['referer']) && is_singular()) {
            include_once AVANS_PATH.'includes/front/actions/share-links.php';
        }
    }

    public function avans_gravity_after_submission($entry, $form)
    {
        include_once AVANS_PATH.'includes/front/actions/gform-submit.php';
    }

    public static function add_score_gform($user,$form,$entry){
        $score = $form['avans_score_value_gravity_form'];
        $repeat = $form['avans_score_repeat_gravity_form'];
        $entry_viewed = gform_get_meta($entry['id'], AVANS_PREFIX.'gform_entry_viewed');
        $user_form_viewed = (array)get_user_meta($user->ID, AVANS_PREFIX.'gform_submited', true);

        if ($repeat != 'yes' && in_array($entry['form_id'], $user_form_viewed)) {
            return false;
        }

        if ((int)$score <= 0 || (int)$user->ID <= 0 || !empty($entry_viewed)) {
            return false;
        }
  
        $details = array(
            array(
                'key' => 'form_id',
                'value' => $entry['form_id'],
            ),
            array(
                'key' => 'entry_id',
                'value' => $entry['id'],
            )
        );

        if (!user::is_permision_get_score('submit_gform', $user->ID)) {
            return false;
        }
        user::add_log($user->ID, 'submit_gform', $details, (int)$score);
        gform_update_meta($entry['id'], AVANS_PREFIX.'gform_entry_viewed', true);
        array_push($user_form_viewed, $entry['form_id']);
        update_user_meta($user->ID, AVANS_PREFIX.'gform_submited', $user_form_viewed);
        do_action('avans_after_gform_submitted', $user, $entry, $form);
        return true;
    }

    public function check_cron_jobs(){
        if (!wp_next_scheduled('avans_daily_event')) {
            wp_schedule_event(time(), 'daily', 'avans_daily_event');
        }
    }

    public function avans_daily_schedule()
    {
        $this->set_birthday_score_users();
        $this->set_registerday_score_users();
        self::check_expired_scores();
    }

    public static function check_expired_scores(){
        global $wpdb;
        $tbl = $wpdb->prefix.user::$table_log;
        $current_time = current_time('mysql');
        $query = "SELECT * FROM {$tbl} WHERE status = 1 AND score > 0 AND expire_date IS NOT NULL AND expire_date <= '$current_time' ";
        $results = $wpdb->get_results($wpdb->prepare($query)); 

        $ids = [];
        $user_score = [];

        foreach((array)$results as $result){

            if(!isset($user_score[(int)$result->user])) {
                $user_score[(int)$result->user] = 0;
            }

            $user_score[(int)$result->user] += (int)$result->score;
            array_push($ids,$result->id);
        }

        foreach($user_score as $user => $score){

            $balance = get_user_meta($user,AVANS_PREFIX.'balance_score',true);
            if($balance >= (int)$score){
                $balance -= (int)$score;
                update_user_meta($user,AVANS_PREFIX.'balance_score',$balance);
            }else{
                $final_score = (int)$score - $balance;
                user::set_score_valid((int)$user, -(int)$final_score);
                update_user_meta($user,AVANS_PREFIX.'balance_score',0);
            }

        }
        
        $ids_str = implode(',',$ids);
        if(count($ids) > 0){
            $wpdb->query("UPDATE $tbl SET status = 0 WHERE id IN ($ids_str)");
        }
    }

    public function set_birthday_score_users(){

        $settings = AVANS_SETTINGS['score_settings'];

        $users = get_users(
            array( 'meta_key' => 'avans_birthdate', 'meta_value' => '', 'meta_compare' => '!=' )
        );

        if ( $settings['birthday']['active'] === true || AVANS_SETTINGS['coupen_settings']['after-birthday']['active'] === 'true' ) {

            foreach ($users as $user) {

                $birthday = get_user_meta($user->ID, AVANS_PREFIX.'birthdate', true);
                $birthday_gettd = (array)get_user_meta($user->ID, AVANS_PREFIX.'birthdate_getted', true);
                $birthday_exp = explode('-', $birthday);

                if (!in_array(date("Y", time()),$birthday_gettd) && (int)date("m", time()) == (int)$birthday_exp[1] && (int)date("d", time()) == (int)$birthday_exp[2]) {

                    if($settings['birthday']['active'] === true){
                        user::add_log($user->ID,'birthday');
                    }

                    if($settings['coupen_settings']['after-birthday']['active'] === 'true'){
                        actions::avans_send_coupon('after-birthday',$user->ID);
                    }

                    update_user_meta($user->ID, AVANS_PREFIX.'birthday_gift_getted',true);
                    array_push($birthday_gettd,date("Y", time()));
                    update_user_meta($user->ID, AVANS_PREFIX.'birthdate_getted', $birthday_gettd);

                    do_action('avans_birthday_user',$user);

                }
                
            }

        }
    }

    public function set_registerday_score_users(){

        $settings = AVANS_SETTINGS['score_settings'];
        $users = get_users();
        if ($settings['registerday']['active'] === true || AVANS_SETTINGS['coupen_settings']['after-registerday']['active'] === 'true' ) {

            foreach ($users as $user) {
                $registerd_date_gettd = (array)get_user_meta($user->ID, AVANS_PREFIX.'registerdate_getted', true);

                if (!in_array(date("Y", time()), $registerd_date_gettd) &&
                    date("m", strtotime($user->user_registered)) === date("m", time()) &&
                    date("d", strtotime($user->user_registered)) === date("d", time()) &&
                    date("Y-m-d", strtotime($user->user_registered)) !== date("Y-m-d", time())) {

                    if( $settings['registerday']['active'] === true ){
                        user::add_log($user->ID, 'registerday');
                    }
                    if( AVANS_SETTINGS['coupen_settings']['after-registerday']['active'] === 'true' ){
                        actions::avans_send_coupon('after-registerday', $user->ID);
                    }
                    array_push($registerd_date_gettd, date("Y", time()));
                    update_user_meta($user->ID, AVANS_PREFIX.'registerdate_getted', $registerd_date_gettd);
                    do_action('avans_after_anniversary_membership',$user);

                }

            }
        }

    }

    public static function avans_send_coupon($event,$user_id=null,object $order=null){

        $data = AVANS_SETTINGS['coupen_settings'][$event];
        if($data['active'] !== 'true'){
            return;
        }

        $coupon_args = new stdClass();
        $coupon_args->char = func::is_set($data['num_char']) ? $data['num_char'] : 4;
        $coupon_args->prefix = func::is_set($data['prefix']) ? $data['prefix'] : '';
        $coupon_args->type = func::is_set($data['type']) ? $data['type'] : 'percent';
        $coupon_args->amount = func::is_set($data['value']) ? $data['value'] : 0;
        $coupon_args->expire = func::is_set($data['expire']) ? $data['expire'] : 10;
        $coupon_args->include = func::is_set($data['limit_products']) ? implode(',',$data['limit_products']) : '';
        $coupon_args->exclude = func::is_set($data['exclude_products']) ? implode(',',$data['exclude_products']) : '';
        $coupon_args->include_cats = func::is_set($data['limit_cats']) ? $data['limit_cats'] : array();
        $coupon_args->exclude_cats = func::is_set($data['exclude_cats']) ? $data['exclude_cats'] : array();
        $coupon_args->min_amount = func::is_set($data['min_amount']) ? $data['min_amount'] : '';
        $coupon_args->max_amount = func::is_set($data['max_amount']) ? $data['max_amount'] : '';
        $coupon_args->email = func::is_set($order) ? $order->get_billing_email() : '';
        $coupon_args->max_discount = func::is_set($data['max_discount']) ? $data['max_discount'] : '';
        $coupon_args->exclude_sale_products = (func::is_set($data['exclude_sale_product']) && $data['exclude_sale_product'] == 'true' ) ? true : false;

        if($coupon_args->amount == 0){
            return;
        }
        
        $coupon_code = func::avans_create_coupon($coupon_args);
        $data['coupon_generated_code'] = $coupon_code;

        $vars_or_text = data::get_text_sms($data,$user_id,$order);
        $phone = data::get_user_mobile($user_id,$order);

        
        if($user_id > 0){
            $result = self::get_coupon_sms_args_for_record_gift($coupon_args,$coupon_code);
            user::add_gift_log(self::get_id_coupon_sms_for_record_gift($event), $user_id, 'completed', $result);
        }

        if(func::is_set($phone) && func::is_set($vars_or_text)){

            $pattern = '';
            if($data['sms']['type'] == 'pattern' || !isset($data['sms']['type'])){
                $pattern = $data['sms']['pattern'];
            }
            do_action('avans_send_sms',$phone,$vars_or_text,$pattern);
            do_action('avans_after_send_coupon',$phone,$vars_or_text,$pattern,$coupon_code);
            if( isset($order) && $order instanceof WC_Order ) {
                $order->add_order_note( "پیامک کد تخفیف یکبار مصرف برای کاربر به شماره $phone ارسال گردید" );
            } 

        }

    }

    public static function get_coupon_sms_name_for_gift_list($id){
        switch($id){
            case -1:
                return 'کد تخفیف هدیه خرید';
            break;
            case -2:
                return 'کد تخفیف هدیه سالگرد تولد';
            break;
            case -3:
                return 'کد تخفیف هدیه سالگرد عضویت';
            break;
            case -4:
                return 'کد تخفیف هدیه ثبت نام';
            break;
            case -5:
                return 'کد تخفیف بابت ارسال دیدگاه در صفحه محصول';
            break;
        }
    }


    public static function get_id_coupon_sms_for_record_gift($event){
        switch($event){
            case 'after-order':
                return -1;
            break;
            case 'after-birthday':
                return -2;
            break;
            case 'after-registerday':
                return -3;
            break;
            case 'after-register':
                return -4;
            break;
            case 'after-comment-product':
                return -5;
            break;
        }
    }


    public static function get_coupon_sms_args_for_record_gift($options,$coupon){

        $data = $options;
        $args = array(
            'char' => $data->char,
            'expire_date' => $data->expire,
            'prefix' => '',
            'suffix' => '',
            'type' => $data->type,
            'amount' => $data->amount,
            'include_product' => $data->include,
            'include_category' => $data->include_cats,
        );
        $data = [
            'code' => $coupon,
            'args' => $args,
        ];
        return $data;
    
    }

}
