<?php namespace Avans;

use Avans\parts\badge;
use Avans\parts\coupons;
use Avans\parts\gifts;
use stdClass;
use WP_Post;
use WP_User_Query;

class data
{
    protected static $log_tbl_score = 'avans_log_score';
    protected static $log_tbl_visit = 'avans_log_visit';
    protected static $log_tbl_meta = 'avans_log_meta';
    private static $log_tbl_refs = 'avans_log_refs';

    public static function is_visit($ip, $page_id)
    {
        global $wpdb;
        $tbl = $wpdb->prefix.self::$log_tbl_visit;
        $query = "SELECT COUNT(*) FROM $tbl WHERE ip='$ip' AND page_id='$page_id' ";
        if ($wpdb->get_var($query) > 0) {
            return true;
        }
        return false;
    }

    public static function add_visit($ip, $page_id, $details=array())
    {
        global $wpdb;
        $tbl = $wpdb->prefix.self::$log_tbl_visit;
        $result = $wpdb->insert($tbl, array(
            'ip' => $ip,
            'page_id' => $page_id,
            'details' => serialize($details),
        ));
        return $result;
    }


    public static function get_lms_postTypes_support(){

        return array(
            'course',
            'lp_course',
            'sfwd-course',
            'stm-courses',
            'courses'
        );

    }



    public static function get_root_data_action($type)
    {
        $score_settings = AVANS_SETTINGS['score_settings'];
        switch ($type) {
            case 'view-post':
                return $score_settings['post'];
            break;
            case 'wc_order':
                return $score_settings['purchase'];
            break;
            case 'view-course':
                return $score_settings['course'];
            break;
            case 'view-product':
                return $score_settings['product'];
            break;
            case 'daily-visit':
                return $score_settings['daily'];
            break;
            case 'share-link':
                return $score_settings['share'];
            break;
            case 'birthday':
                return $score_settings['birthday'];
            break;
            case 'comment':
                return $score_settings['comment'];
            break;
            case 'publish-post':
                return $score_settings['publish'];
            break;
            case 'publish-product':
                return $score_settings['publish-product'];
            break;
            case 'post-rating':
                return $score_settings['post-rating'];
            break;
            case 'registerday':
                return $score_settings['registerday'];
            break;
            case 'register':
                return $score_settings['register'];
            break;
            case 'complete-profile':
                return $score_settings['complete-profile'];
            break;
            case 'ref-register':
                return $score_settings['ref-register'];
            break;
            case 'lms_course_complete':
                return $score_settings['course']['complete_course'];
            break;
            case 'lms_course_enrolled':
                return $score_settings['course']['enroll_course'];
            break;
            case 'lms_lesson_complete':
                return $score_settings['course']['complete_lesson'];
            break;
            case 'lms_quiz_passed':
                return $score_settings['course']['passed_test'];
            break;
            default:
                return $score_settings['unname'];
            break;
        }
    }


    public static function remove_user_actions($user_id){
        global $wpdb;
        $log_tbl = $wpdb->prefix . user::$table_log;
        $log_tbl_meta = $wpdb->prefix . user::$tabel_log_meta;
        $results = $wpdb->get_results( $wpdb->prepare("SELECT id,user FROM $log_tbl WHERE user=%d", $user_id) );
        $logs = [];
        foreach($results as $result){
            $logs[] = $result->id;
        }
        $ids_str = implode(',',$logs);
        $wpdb->delete( $log_tbl, array( 'user' => $user_id ) );
        $wpdb->query("DELETE FROM $log_tbl_meta WHERE log_id IN ($ids_str)");
    }


    public static function get_levels()
    {
        $data = get_option('avans_plugin_setting', []);
        $levels_array = !empty($data['avans_user_levels']) ? $data['avans_user_levels'] : [['level'=>'','min_score'=>'']];
        $levels = array_map('self::level_to_object', $levels_array);
        usort( $levels, 'self::level_comparator' );

        return $levels;
    }

    public static function level_to_object($data)
    {
        $class_object = new stdClass();
        $class_object->name = $data['level'];
        $class_object->score = (int)$data['min_score'];
        return $class_object;
    }

    public static function level_comparator($object1, $object2)
    {
        return $object1->score < $object2->score;
    }

    public static function check_condition($conditions, $cond, $data)
    {
        foreach ((array)$conditions as $condition) {
            $status = false;
            switch ((string)$condition['operation']) {
                case '==':
                    //for check badge
                    if(is_array($data)){
                        $status = ($condition['condition'] == $cond && in_array($condition['data'],$data) ) ?  true : false;
                        break;
                    }
                    // for check role or level
                    $status = ($condition['condition'] == $cond && $condition['data'] == $data) ?  true : false;
                break;
                case '!=':
                    //for check badge
                    if(is_array($data)){
                        $status = ($condition['condition'] == $cond && !in_array($condition['data'],$data) ) ?  true : false;
                        break;
                    }
                    // for check role or level
                    $status = ($condition['condition'] == $cond && $condition['data'] != $data) ?  true : false;
                break;
            }

            if ($status) {
                return $condition['score'];
            }
        }

        return null;
    }

    public static function get_action_score($action,$user_id, $customScore = null)
    {

        $level = user::get_level($user_id);
        $role =  user::get_role($user_id);
        $badges = user::get_badges_user($user_id);

        $root = self::get_root_data_action($action);

        $normal = isset($customScore) ? $customScore : (int)$root['score'];

        $role = (int)self::check_condition((array)$root['condition'], 'role', $role);
        $level = (int)self::check_condition((array)$root['condition'], 'level', $level);
        $badge = (int)self::check_condition((array)$root['condition'], 'badge', $badges);


        if(max($role, $level,$badge) > 0){
            $scoreRole = $normal * $role / 100;
            $scoreLevel = $normal * $level / 100;
            $scoreBadge = $normal * $badge / 100;
            return (int)max($scoreRole , $scoreLevel , $scoreBadge);
        }

        return $normal;
    }

    public static function get_logs(array $args=null)
    {
        global $wpdb;
        $table_log = $wpdb->prefix.self::$log_tbl_score;
        $table_meta = $wpdb->prefix.self::$log_tbl_meta;
        $sql = "";
        $where = "";
        $where_clauses = [];

        if (isset($args['sort'])) {
            $sort = $args['sort'];
            $order_by = $args['order_by'];
            $sql .= " ORDER BY $order_by $sort ";
        } else {
            $sql .= " ORDER BY id DESC ";
        }

        $page = isset($args['page']) ? (int)$args['page'] : 1;
        $per_page = isset($args['per_page']) ? $args['per_page'] : 10;
        $offset = ($page * $per_page) - $per_page;
        $sql .= " LIMIT $offset, $per_page ";

        if (isset($args['where']) && $args['where']) {
            $where .= " WHERE ";
        }

        if (isset($args['include'])) {
            $include = $args['include'];
            if (isset($include['type'])) {
                $type = $include['type'];
                $query = " type IN ('$type') ";
                array_push($where_clauses, $query);
            }
            
            if (isset($include['user'])) {
                $users = $include['user'];
                $query = " user IN ($users) ";
                array_push($where_clauses, $query);
            }
        }

        if (isset($args['exclude'])) {
            $exclude = $args['exclude'];
            if (isset($exclude['type'])) {
                $type = $exclude['type'];
                $query = " type NOT IN ('$type') ";
                array_push($where_clauses, $query);
            }
            
            if (isset($exclude['user'])) {
                $users = $exclude['user'];
            }
        }

        if (isset($args['time'])) {
            $from = $args['time']['from'];
            $to = $args['time']['to'];
            if (isset($from)) {
                $query = " created_at >= '$from' ";
                array_push($where_clauses, $query);
            }
            if (isset($to)) {
                $query = " created_at <= '$to' ";
                array_push($where_clauses, $query);
            }
        }

        if (count($where_clauses) > 0) {
            foreach ($where_clauses as $index => $where_clause) {
                if ($index>0) {
                    $where .= 'AND'.$where_clause;
                } else {
                    $where .= $where_clause;
                }
            }
        }
        $query = "SELECT * FROM {$table_log} $where";

        $total= $wpdb->get_var("SELECT COUNT(1) FROM ($query) AS combined_table");
        $total_page = ceil($total / $per_page);

        $results  = $wpdb->get_results($query.$sql);
        $search = [
            'query' => $query,
            'total' => $total,
            'page' => $page,
            'total_page' => $total_page,
            'result' => $results
        ];

        return $search;
    }

    public static function get_logs_gifts(array $args=null)
    {
        global $wpdb;
        $table_log = $wpdb->prefix.user::$tabel_log_gift;
        $sql = "";
        $where = "";
        $where_clauses = [];

        if (isset($args['sort'])) {
            $sort = $args['sort'];
            $order_by = $args['order_by'];
            $sql .= " ORDER BY $order_by $sort ";
        } else {
            $sql .= " ORDER BY id DESC ";
        }

        $page = isset($args['page']) ? (int)$args['page'] : 1;
        $per_page = isset($args['per_page']) ? $args['per_page'] : 10;
        $offset = ($page * $per_page) - $per_page;
        $sql .= " LIMIT $offset, $per_page ";

        if (isset($args['where']) && $args['where']) {
            $where .= " WHERE ";
        }

        if (isset($args['time'])) {
            $from = $args['time']['from'];
            $to = $args['time']['to'];
            if (isset($from)) {
                $query = " create_at >= '$from' ";
                array_push($where_clauses, $query);
            }
            if (isset($to)) {
                $query = " create_at <= '$to' ";
                array_push($where_clauses, $query);
            }
        }

        if (isset($args['include'])) {
            $include = $args['include'];            
            if (isset($include['user'])) {
                $users = $include['user'];
                $query = " user IN ($users) ";
                array_push($where_clauses, $query);
            }
            if (isset($include['gift'])) {
                $users = $include['gift'];
                $query = " gift IN ($users) ";
                array_push($where_clauses, $query);
            }
            
        }

        if (count($where_clauses) > 0) {
            foreach ($where_clauses as $index => $where_clause) {
                if ($index>0) {
                    $where .= 'AND'.$where_clause;
                } else {
                    $where .= $where_clause;
                }
            }
        }
        $query = "SELECT * FROM {$table_log} $where";

        $total= $wpdb->get_var("SELECT COUNT(1) FROM ($query) AS combined_table");
        $total_page = ceil($total / $per_page);

        $results  = $wpdb->get_results($query.$sql);
        $search = [
            'query' => $query,
            'total' => $total,
            'page' => $page,
            'total_page' => $total_page,
            'result' => $results
        ];

        return $search;
    }


    public static function get_logs_refs(array $args=null)
    {
        global $wpdb;
        $table_log = $wpdb->prefix.self::$log_tbl_refs;
        $sql = "";
        $where = "";
        $where_clauses = [];

        if (isset($args['sort'])) {
            $sort = $args['sort'];
            $order_by = $args['order_by'];
            $sql .= " ORDER BY $order_by $sort ";
        } else {
            $sql .= " ORDER BY id DESC ";
        }

        $page = isset($args['page']) ? (int)$args['page'] : 1;
        $per_page = isset($args['per_page']) ? $args['per_page'] : 10;
        $offset = ($page * $per_page) - $per_page;
        $sql .= " LIMIT $offset, $per_page ";

        if (isset($args['where']) && $args['where']) {
            $where .= " WHERE ";
        }

        if (isset($args['time'])) {
            $from = $args['time']['from'];
            $to = $args['time']['to'];
            if (isset($from)) {
                $query = " created_at >= '$from' ";
                array_push($where_clauses, $query);
            }
            if (isset($to)) {
                $query = " created_at <= '$to' ";
                array_push($where_clauses, $query);
            }
        }

        if (isset($args['include'])) {
            $include = $args['include'];            
            if (isset($include['user'])) {
                $users = $include['user'];
                $query = " ref_id IN ($users) ";
                array_push($where_clauses, $query);
            }
        }

        if (count($where_clauses) > 0) {
            foreach ($where_clauses as $index => $where_clause) {
                if ($index>0) {
                    $where .= 'AND'.$where_clause;
                } else {
                    $where .= $where_clause;
                }
            }
        }
        $query = "SELECT * FROM {$table_log} $where";

        $total= $wpdb->get_var("SELECT COUNT(1) FROM ($query) AS combined_table");
        $total_page = ceil($total / $per_page);

        $results  = $wpdb->get_results($query.$sql);
        $search = [
            'query' => $query,
            'total' => $total,
            'page' => $page,
            'total_page' => $total_page,
            'result' => $results
        ];

        return $search;
    }

    public static function get_users(array $args=null)
    {
        // return $args;
        $users = array();
        $result = array();
        $user_query = array();
        $total = 0;
        global $wpdb;
        $table_log = $wpdb->prefix.self::$log_tbl_score;
        $query = '';


        $page = isset($args['page']) ? (int)$args['page'] : 1;
        $per_page = isset($args['per_page']) ? (int)$args['per_page'] : 10;
        $offset = ($page * $per_page) - $per_page;

        if (!isset($args['where']) || !$args['where']) {
            $arguments = array(
            'paged' => $page,
            'number' => $per_page,
            'offset' => $offset,
            'meta_key' => AVANS_PREFIX.'user_score',
            'orderby' => 'meta_value_num',
            'order' => 'DESC',
        );

            $user_query = new \WP_User_Query($arguments);
            $users = $user_query->get_results();
            $total = $user_query->get_total();
        }

        if (isset($args['user'])) {
            $users = [ 0=> get_user_by('ID',$args['user']) ];
        }

        
        if (isset($args['type']) || isset($args['time'])) {
            $where_clauses = array();
            $where = "";

            if (isset($args['time'])) {
                $from = $args['time']['from'];
                $to = $args['time']['to'];
                if (isset($from)) {
                    $query = " created_at >= '$from' ";
                    array_push($where_clauses, $query);
                }
                if (isset($to)) {
                    $query = " created_at <= '$to' ";
                    array_push($where_clauses, $query);
                }
            }

            if (isset($args['type'])) {
                $type = $args['type'];
                $query = " type IN ('$type') ";
                array_push($where_clauses, $query);
            }


            if (count($where_clauses) > 0) {
                foreach ($where_clauses as $index => $where_clause) {
                    if ($index>0) {
                        $where .= 'AND'.$where_clause;
                    } else {
                        $where .= $where_clause;
                    }
                }
            }

            $sql = " LIMIT $offset, $per_page ";
            if( !isset($args['type']) ){
                $query = "SELECT a.user as ID,SUM(a.score) as score FROM (SELECT * FROM $table_log WHERE ( score > 0 || type IN ('admin') ) AND $where ) as a  GROUP BY user ORDER BY score DESC";
            }else{
                $query = "SELECT a.user as ID,SUM(a.score) as score FROM (SELECT * FROM $table_log WHERE score > 0 AND $where ) as a  GROUP BY user ORDER BY score DESC";
            }
            $total= $wpdb->get_var("SELECT COUNT(1) FROM ($query) AS combined_table");
            $users  = $wpdb->get_results($query.$sql);
        }

        foreach ($users as $index => $user) {
            $result[] = array(
                'rank' => self::get_user_rank((int)$index, (int)$per_page, (int)$page),
                'user' => $user->ID,
                'is_filter' => (isset($args['time']) || isset($args['type'])) ? true: false,
                'filter_score' => (int)$user->score,
                'score' => user::get_score((int)$user->ID),
                'score_valid' => user::get_score_valid((int)$user->ID),
                'ref_code' => user::get_ref_code((int)$user->ID)
            );
        }

        $total_page = ceil($total / $per_page);
        
        $search = [
            'query' => $query,
            'total' => $total,
            'page' => $page,
            'total_page' => $total_page,
            'result' => $result,
        ];

        return $search;
    }

    public static function get_user_rank(int $index, int $per_page, int $page = null)
    {
        $rank = 0;
        if (isset($page)):
            $prevNum = ($page - 1) * $per_page;
        $rank = $prevNum + $index + 1; else:
            $rank = $index + 1;
        endif;

        return $rank;
    }

    public static function get_users_search($array)
    {
        $args = array(
            'search' => '*'.esc_attr($array['user']).'*',
            'search_columns' => array( 'user_login', 'user_login', 'user_nicename', 'user_email'),
        );
        $user_query = new \WP_User_Query($args);
        $userId = [];
        if (!empty($user_query->results)) {
            foreach ($user_query->results as $index=>$user) {
                array_push($userId, $user->ID);
            }
        }
        $user_list = implode(',', $userId);
        return $user_list;
    }


    public static function get_description_permision_convert(){
        $settings = AVANS_SETTINGS['convert_settings']['wallet']['limit'];
        switch($settings['unit']){
            case 'day':
                $unit = 'روز';
            break;
                  case 'week':
                $unit =  'هفته';
            break;
                  case 'month':
                $unit =  'ماه';
            break;
                  case 'year':
                $unit =  'سال';
            break;
        }
        $text = '<p>'.sprintf(__('شما نمی توانید بیشتر از %s بار در %s %s را به کیف پول تبدیل کنید.'),$settings['value'],$unit,func::get_score_label(true)).'</p>';
        $description = '<p>باتوجه به اینکه شما از سقف این میزان استفاده کرده اید برای تبدیل مجدد باید روز دیگر تلاش کنید.</p>';
        return $text.$description;
    }

    public static function get_text_sms($data,$user_id=null,$order=null,$otp=null){

        $vars = $data['sms']['vars'];
        $text = $data['sms']['text'];
        $type = $data['sms']['type'];



        $coupon = $data['coupon_generated_code'];

        $fullname = func::is_set($order) ? $order->get_billing_first_name() .' '.$order->get_billing_last_name() : '';

        if(!func::is_set($fullname) && (int)$user_id > 0){
            $data = user::get_user_info($user_id);
            $fullname = $data->display_name;
            if(func::is_set($data->firstname) || func::is_set($data->lastname)){
                $fullname = $data->firstname . ' ' .$data->lastname;
            }
        }

        if (is_array($vars) && (!isset($type) || $type == 'pattern') ) {
            $array = [];
            foreach ($vars as $var) {
                $value = $var['value'];
                if (func::is_set($coupon)) {
                    $value = str_replace('{coupon}', $coupon, $value);
                } 
                $value = str_replace('{display_name}', $fullname, $value);
                if((int)$user_id > 0){
                    $value = str_replace('{score}',user::get_score_valid($user_id), $value);
                }else{
                    $value = str_replace('{score}','', $value);
                }
                if (func::is_set($otp)) {
                    $value = str_replace('{OTP}', $otp, $value);
                }
                $array[$var['key']] = $value;
            }
            return $array;
        }else{
            $value = $text;
            if (func::is_set($coupon)) {
                $value = str_replace('{coupon}', $coupon, $value);
            }
            $value = str_replace('{display_name}', $fullname, $value);
            if((int)$user_id > 0){
                $value = str_replace('{score}', user::get_score_valid($user_id), $value);
            }else{
                $value = str_replace('{score}', '', $value);
            }
            if(func::is_set($otp)){
                $value = str_replace('{OTP}', $otp , $value);
            }
            return $value;
        }


        
    }

    public static function get_user_mobile($user_id , $order=null){

        if(func::is_set($order)){
            $mobile = $order->get_billing_phone();
            if(func::is_set($mobile)){
                return $mobile;
            }
        }

        if((int)$user_id <= 0){
            return;
        }

        $user_mobile = get_user_meta($user_id, 'billing_phone', true);

        if (func::is_set($user_mobile)) {
            return $user_mobile;
        }

        $digits_phone = get_user_meta($user_id,'digits_phone',true);
        if (func::is_set($digits_phone)) {
            return $digits_phone;
        }

        $wupp_mobile = get_user_meta($user_id, 'wupp_mobile', true);
        if (func::is_set($wupp_mobile)) {
            $wupp_country_code = get_user_meta($user_id, 'wupp_country_code', true);
            return $wupp_country_code.$wupp_mobile;
        }

        $wpyarud_phone = get_user_meta($user_id, 'wpyarud_phone', true);

        if (func::is_set($wpyarud_phone)) {
            return $wpyarud_phone;
        }


        return null;
    }



    public static function get_log_title($type, $args)
    {
        $title = '';
        switch ($type) {
            case 'view-post':
                $title = sprintf(__('مشاهده مطلب %s', 'avans_plugin'), get_the_title($args['post-id']));
                break;
            case 'view-course':
                $title = sprintf(__('مشاهده دوره %s', 'avans_plugin'), get_the_title($args['post-id']));
            break;
            case 'view-product':
                $title = sprintf(__('مشاهده محصول %s', 'avans_plugin'), get_the_title($args['post-id']));
                break;
            case 'view-page':
                $title = sprintf(__('مشاهده صفحه %s', 'avans_plugin'), get_the_title($args['post-id']));
            break;
            case 'lms_course_complete':
                $title = sprintf(__('تکمیل دوره  %s', 'avans_plugin'), get_the_title($args['course_id']));
            break;
            case 'lms_course_enrolled':
                $title = sprintf(__('ثبت نام در دوره  %s', 'avans_plugin'), get_the_title($args['course_id']));
            break;
            case 'lms_lesson_complete':
                $title = sprintf(__('تکمیل درس  %s در دوره %s', 'avans_plugin'), get_the_title($args['lesson_id']), get_the_title($args['course_id']) );
            break;
            case 'lms_quiz_passed':
                $title = sprintf(__('قبولی در آزمون %s در دوره %s', 'avans_plugin'), get_the_title($args['quiz_id']) , get_the_title($args['course_id']) );
            break;
            case 'wc_order':
                $title = sprintf(__('خرید از سایت به شماره سفارش %d', 'avans_plugin'),$args['order_id']);
                break;
            case 'wc_order_cancelled':
                $title = sprintf(__('لغو امتیاز خرید از سایت به شماره سفارش %d', 'avans_plugin'),$args['order_id']);
                break;
            case 'daily-visit':
                $title = __('مشاهده روزانه', 'avans_plugin');
                break;
            case 'view-video':
                $title = __('مشاهده ویدئو %s در صفحه %s', 'avans_plugin');
                $title = sprintf(__('مشاهده ویدئو به شناسه %s در صفحه %s', 'avans_plugin'),$args['object_id'],get_the_title($args['post_id']));
                break;
            case 'view-sound':
                $title = sprintf(__('گوش کردن پادکست به شناسه %s در صفحه %s', 'avans_plugin'),$args['object_id'],get_the_title($args['post_id']));
                break;
            case 'convert-wallet':
                $title = sprintf(__('تبدیل %s به اعتبار کیف پول', 'avans_plugin'),func::get_score_label());
                break;
            case 'register':
                $title = __('ثبت نام در سایت', 'avans_plugin');
                break;
            case 'publish-post':
                $title = sprintf(__(' انتشار مطلب  %s', 'avans_plugin'), get_the_title($args['post_id']));
                break;
            case 'publish-product':
                $title = sprintf(__(' انتشار محصول  %s', 'avans_plugin'), get_the_title($args['post_id']));
            break;
            case 'comment':
                $title = sprintf(__(' قراردادن دیدگاه در %s', 'avans_plugin'), get_the_title($args['post_id']));
                break;
            case 'share-link':
                $title = sprintf(__('اشتراک گذاری صفحه %s', 'avans_plugin'), get_the_title($args['post_id']));
                break;
            case 'birthday':
                $title = __('به مناسبت سالگرد تولد', 'avans_plugin');
            break;
            case 'registerday':
                $title = __('به مناسبت سالگرد عضویت', 'avans_plugin');
                break;
            case 'admin':
                $title = sprintf(__('اهدا %s مدیریت %s', 'avans_plugin'),func::get_score_label(),$args['title']);
            break;
            case 'gifts':
                $title = sprintf(__('دریافت جایزه %s', 'avans_plugin'), $args['gift-id']);
            break;
            case 'spin-wheel':
                $title = sprintf(__('چرخاندن گردونه شانس %s', 'avans_plugin'), get_the_title($args['wheel-id']));
            break;
            case 'spin_wheel_score':
                $title = sprintf(__('%s هدیه (از طریق گردونه شانس %s)', 'avans_plugin'),func::get_score_label(),get_the_title($args['wheel-id']));
            break;
             case 'lottery':
                $title = sprintf(__('شرکت در قرعه کشی %s', 'avans_plugin'), get_the_title($args['lottery-id']));
            break;
            case 'post-rating':
                $title = sprintf(__('ستاره دادن به مطلب %s', 'avans_plugin'), get_the_title($args['post-id']));
            break;
            case 'wc_referer':
                $title = sprintf(__('دریافت پورسانت معرفی بابت سفارش %d', 'avans_plugin'), $args['order_id']);
            break;
            case 'submit_gform':
            $title = sprintf(__('تکمیل فرم %s به شناسه ورودی %s', 'avans_plugin'), func::get_gform_title($args['form_id']) , $args['entry_id'] );
            break;
            case 'complete-profile':
            $title = sprintf(__('تکمیل پروفایل کاربری', 'avans_plugin'), func::get_gform_title($args['form_id']) , $args['entry_id'] );
            break;
            case 'import_form_plugins':
                $title = __('درون ریزی امتیازهای قدیمی', 'avans_plugin');
            break;
            case 'ref-register':
                if ((int)$args['user-id'] > 0) {
                    $user = get_user_by('ID', $args['user-id']);
                    $title = sprintf(__('معرفی و ثبت نام کاربر جدید (%s)', 'avans_plugin'), $user->display_name);
                }else{
                    $title = sprintf(__('معرفی و ثبت نام کاربر جدید', 'avans_plugin'));
                }
            break;
        }
        return apply_filters("avans_get_log_title",$title,$type,$args);
    }

    public static function get_string_time($time_key){
        switch($time_key){
            case "hour":
                $text = "ساعت";
                break;
            case "day":
                $text = "روز";
                break;
            case "week":
                $text = "هفته";
                break;
            case "month":
                $text = "ماه";
                break;
            case "year":
                $text = "سال";
            break;
        }
        return $text;
    }

    public static function avans_is_valid_date($string){
        $parts = explode('/',$string);
        $year = (int)avans_tr_num($parts[0]);
        $month = (int)avans_tr_num($parts[1]);
        $day = (int)avans_tr_num($parts[2]);

        if (
        $day <= 0 ||
        $day > 31 ||
        $month <= 0 ||
        $month > 12 ||
        $year < 1300 ||
        $year > 3000
        ) {
        return false;
        }
        return true;
    }

    public static function get_meta_single_log($log_id)
    {
        global $wpdb;
        $table_meta = $wpdb->prefix.self::$log_tbl_meta;
        $meta = [];
        $results = $wpdb->get_results("SELECT * FROM  $table_meta WHERE log_id = $log_id");
        if (count($results) > 0) {
            foreach ($results as $result) {
                $meta[(string)$result->meta_key] = $result->meta_value;
            }
        }

        return $meta;
    }

    public static function get_gifts(array $args=array())
    {
        if (empty($args)) {
            $args = [
                'post_type'        => 'avans-gift',
                'posts_per_page'  => 12,
            ];
        }
        $posts = new \WP_Query($args);
        $posts->current_page = isset($args['paged']) ? $args['paged'] : 1;
        return $posts;
    }


    public static function get_spinwheels(array $args=array())
    {
        if (empty($args)) {
            $args = [
                'post_type'        => 'avans-spin-wheel',
                'posts_per_page'  => 12,
            ];
        }
        $posts = new \WP_Query($args);
        $posts->current_page = isset($args['paged']) ? $args['paged'] : 1;
        return $posts;
    }

    public static function get_type_gifts($content){
        if(strpos($content, 'code') !== false){
            return array(
                'type'  => 'coupon',
                'title' => 'کد تخفیف',
            );
        }
        if(strpos($content, 'order_id') !== false){
            return array(
                'type'  => 'product',
                'title' => 'محصول',
            );
        }
        if(strpos($content, 'file') !== false){
            return array(
                'type'  => 'file',
                'title' => 'فایل دانلودی',
            );
        }
        if(strpos($content, 'zero') !== false){
            return array(
                'type'  => 'zero',
                'title' => 'پوچ',
            );
        }
        if(strpos($content, 'score') !== false){
            return array(
                'type'  => 'score',
                'title' => 'امتیاز',
            );
        }
         
        return array();        
    }

    public static function get_badges($exclude)
    {
        $args = [
            'post_type'        => 'avans-badge',
            'posts_per_page'  => -1,
        ];

        if (count($exclude) > 0) {
            $args['post__not_in'] = $exclude;
        }

        $posts = new \WP_Query($args);

        return $posts;
    }

    public static function get_gift_meta($post_id)
    {
        $meta = get_post_meta($post_id, AVANS_PREFIX.'gift_settings', true);
        return $meta;
    }

    public static function is_permision_get_gift($user_id,$score)
    {
        $user_score = user::get_score_valid((int)$user_id);
        $calc = (int)$user_score - (int)$score;
        if($calc >= 0){
            return true;
        }else{
            return false;
        }
    }

    public static function get_remains_text_gift($user_id,$score)
    {
        $user_score = user::get_score_valid((int)$user_id);
        $calc = (int)$user_score - (int)$score;
        if ($calc >= 0) {
            return __('همین الان دریافت کنید', 'avans_plugin');
        } else {
            return sprintf(__('%d %s بیشتر نیاز دارید', 'avans_plugin'), abs($calc),func::get_score_label());
        }
    }

    public static function create_coupon($data)
    {
        $coupon = new coupons($data);
        return $coupon->create($data);
    }

    public static function get_payment_unit()
    {
        return function_exists('get_woocommerce_currency_symbol') ? get_woocommerce_currency_symbol(get_option('woocommerce_currency')) : 'تومان';
    }

    public static function get_star_rate_post($post_id)
    {
        $rate = get_post_meta($post_id, AVANS_PREFIX.'post-rate', true);
        return (float)$rate;
    }

    public static function get_log_star_rate_post($post_id)
    {
        $log = (array)get_post_meta($post_id, AVANS_PREFIX.'post-rate-log', true);
        return self::normalize_array($log);
    }

    

    public static function get_expire_time($settings_route,$score,$expire = array()){

        if(empty($expire)){
            $expire = $settings_route['expire'];
        }

        $unit = 60*60*24;

        if(empty($expire['value']) || (int)$expire['value'] == 0 || $score <=0){
            return null;
        }

        switch($expire['unit']){
            case 'year':
                $timeUnit = $unit * 365;
            break;
            case 'month':
                $timeUnit = $unit * 30;
            break;
            case 'week':
                $timeUnit = $unit * 7;
            break;
            case 'day':
                $timeUnit = $unit;
            break;
        }

        $exptime = $timeUnit * (int)$expire['value'];

        $timestamp = time() + $exptime;
        $datetimeFormat = 'Y-m-d H:i:s';
        $date = new \DateTime('now', wp_timezone());
        $date->setTimestamp($timestamp);

        return $date->format($datetimeFormat);

    }

    
    public static function get_average_rate_post($logs)
    {
        $sum = array_sum($logs);
        $avg = $sum / count($logs);
        return $avg;
    }

    public static function set_star_rate_post_by_guest($post_id, $rate){

        $logs = self::get_log_star_rate_post($post_id);
        $key = 'g-'.rand(1111111,9999999);
        $logs[$key] = $rate;
        $avg = self::get_average_rate_post($logs);
        update_post_meta($post_id, AVANS_PREFIX.'post-rate-log', $logs);
        update_post_meta($post_id, AVANS_PREFIX.'post-rate', $avg);
        data::guest_set_post_rated($post_id);
        return [
            'avg' => $avg,
            'score' => 0,
        ];

    }

    public static function set_star_rate_post($post_id, $rate, $user_id)
    {
        $logs = self::get_log_star_rate_post($post_id);

        if (empty($logs[$user_id])) {
            $details = [['key' => 'post-id','value' => $post_id]];
            if (!user::is_permision_get_score('post-rating', $user_id)) {
                return;
            }
            $score = user::add_log($user_id, 'post-rating', $details);
        }
        $logs[$user_id] = $rate;
        $avg = self::get_average_rate_post($logs);
        update_post_meta($post_id, AVANS_PREFIX.'post-rate-log', $logs);
        update_post_meta($post_id, AVANS_PREFIX.'post-rate', $avg);
        $res = [
            'avg' => $avg,
            'score' => $score,
        ];
        return $res;
    }

    public static function get_data_star_rate_post($post_id, $user_id)
    {
        $data = new stdClass;
        $logs = self::get_log_star_rate_post($post_id);
        $data->rate = self::get_star_rate_post($post_id);
        $data->use = !empty($logs[$user_id]) ? true : false;
        $data->count =  count(self::normalize_array($logs));
        return $data;
    }

    public static function normalize_array(array $array)
    {
        $array = array_filter($array, function ($value) {
            return !is_null($value) && $value !== '';
        });

        return $array;
    }

    public static function check_smspanel_pattern_support($panel_name)
    {
        $pattern_support = [
            'farazsms' , 'ippanel' , 'melipayamak' , 'smsir' , 'popupsms' , 'rangine' , 'maxsms' ,'ghasedak', 'kavenegar'
        ];
        if (in_array($panel_name, $pattern_support)) {
            return true;
        }
        return false;
    }

    public static function get_sum_score_for_purchase(int $user_id ,object $order = null,object $cart = null){
        $settings = AVANS_SETTINGS['score_settings']['purchase'];
        $sum_score = 0;
        $total = isset($order) ? self::get_total_order($order) : self::get_total_cart($cart);
        if(!$settings['active'] || $total < (int)$settings['min_price'] ){
            return $sum_score;
        }

        if (isset($order) || isset($cart)) {
            switch ($settings['type']) {
                case 'custom_product_score':
                    $items = isset($order) ? $order->get_items() : $cart->get_cart();
                    foreach ($items as $index => $values) {
                        $item = isset($order) ? $values : $values['data'];
                        $product = isset($order) ? $item->get_product() : $item;
                        if ($product->get_type() == 'variation') {
                            $variation_id = $item->get_variation_id();
                            $postScore = get_post_meta($variation_id, AVANS_PREFIX.'product_score_var', true);
                            $score = !empty($postScore) ? (int)$postScore : 0;
                        } else {
                            $product_id = isset($order) ? $item->get_product_id() : $item->get_id();
                            $postScore = get_post_meta($product_id, AVANS_PREFIX.'product_score', true);
                            $score = !empty($postScore) ? $postScore : 0;
                        }
                        $quantity = isset($order)  ? $item->get_quantity() : $values['quantity'];
                        $score_item = $score * $quantity;
                        $sum_score += $score_item;
                    }

                break;
                case 'volume_purchase':
                    $price = $total;
                    $flex_unit  = !empty($settings['flex_unit']) ?  (int)$settings['flex_unit'] : 1000;
                    $flex_score = !empty($settings['flex_score']) ? (int)$settings['flex_score'] : 1;
                    $sum_score = $flex_score * ($price / $flex_unit);

                break;
                case 'fixed_score':
                    $sum_score = !empty($settings['fix_score']) ? (int)$settings['fix_score'] : 1;
                break;
                case 'fixed_score_by_product':
                    $items = isset($order) ? $order->get_items() : $cart->get_cart();
                    $score = !empty($settings['fix_score_product']) ? (int)$settings['fix_score_product'] : 1;

                    foreach ($items as $index => $values) {
                        $item = isset($order) ? $values : $values['data'];
                        $quantity = isset($order)  ? $item->get_quantity() : $values['quantity'];
                        $score_item = $score * $quantity;
                        $sum_score += $score_item;
                    }

                break;
            }
        }

        $sum_score = data::get_action_score('wc_order', $user_id , $sum_score );

        return (int)$sum_score;
    }

    public static function guest_is_post_rated($post_id){
        $posts = $_COOKIE['avans_posts_rated'];
        $posts = explode('-',$posts);
        if( in_array( $post_id ,$posts)  ){
            return true;
        }
        return false;
    }

    public static function guest_set_post_rated($post_id){
        $posts = $_COOKIE['avans_posts_rated'];
        $posts = explode('-',$posts);
        array_push($posts,$post_id);
        setcookie('avans_posts_rated', implode('-',$posts) , (time() +  86400 * 365 * 20), "/");
    }



    public static function get_total_order($order){
        return (int)number_format( (float) $order->get_total() - $order->get_total_tax() - $order->get_shipping_total() - $order->get_shipping_tax(), wc_get_price_decimals(), '.', '' );
    }
    public static function get_total_cart($cart){
        return (int)number_format( (float) $cart->total - $cart->get_shipping_total() , wc_get_price_decimals(), '.', '' );
    }

    public static function get_single_product_score($user_id,$product_id,$variation_id=null,$qty = 1){
        $score = 0;
        $settings = AVANS_SETTINGS['score_settings']['purchase'];

        if(!$settings['active'] || $settings['type'] == 'fixed_score'){
            return $score;
        }

        $mainproduct = wc_get_product($product_id);
        $price = (int)$mainproduct->get_price();

        if(isset($variation_id) && (int)$variation_id !=0 ){
            $product = wc_get_product($variation_id);
            $price = (int)$product->get_price();
        }


        $cartTotal =  data::get_total_cart(WC()->cart);
        $finalPrice = (int)$qty * $price;

        if( ($cartTotal + $finalPrice) < (int)$settings['min_price'] ){
            return $score;
        }
        
        $permision = user::is_permision_get_score( 'wc_order', $user_id );

        if (!$permision) {
            return $score;
        }


        switch ($settings['type']) {
                case 'custom_product_score':

                    if($mainproduct->is_type('variable')){
                        $score =  (int)get_post_meta($variation_id, AVANS_PREFIX.'product_score_var', true);
                    }else{
                        $score =  (int)get_post_meta($product_id, AVANS_PREFIX.'product_score', true);
                    }
                    
                break;
                case 'volume_purchase':
                    $flex_unit  = !empty($settings['flex_unit']) ?  (int)$settings['flex_unit'] : 1000;
                    $flex_score = !empty($settings['flex_score']) ? (int)$settings['flex_score'] : 1;
                    $score = $flex_score * ($price / $flex_unit);
                break;
                // case 'fixed_score':
                //     $score = !empty($settings['fix_score']) ? (int)$settings['fix_score'] : 1;
                // break;
                case 'fixed_score_by_product':
                    $score = !empty($settings['fix_score_product']) ? (int)$settings['fix_score_product'] : 1;
                break;
            }


            $sum_score = data::get_action_score('wc_order', $user_id, $score);
                
            if(isset($qty)){
                $sum_score = $sum_score * (int)$qty;
            }
            $score = user::get_valid_score($user_id,$sum_score,true);
            return $score;

    }

    public static function is_validate_score($type){
        $types = ['wc_order','comment'];
        if(in_array($type,$types)){
            return true;
        }
        return false;
    }

    public static function get_final_action_score($action, $user_id, $score){
        $score = data::get_action_score($action, $user_id, $score);
        $score = user::get_valid_score($user_id, $score);
        return $score;
    }

    public static function is_permision_send_coupon_by_time($limit_time_value, $limit_time_unit, $last_time_send){
        $unittime = func::covert_string_time_to_timestamp($limit_time_unit);
        $timeneed = (int)$unittime * (int)$limit_time_value;
        $now = time();
        if(($now - $last_time_send) > $timeneed){
            return true;
        }
        return false;
    }


    public static function post_is_commented($comment){
        $args = array(
            'status'=> 'approve',
            'post_id' => $comment->comment_post_ID,
            'user_id'=> $comment->user_id,
        );
        $userComment = get_comments( $args );
        // func::log($userComment);
        if(count($userComment) > 1){
            return true;
        }
        return false;
    }

    public static function get_list_actions(){
        return array(
            'ref-register',
            'register',
            'registerday',
            'publish-product',
            'publish-post',
            'birthday',
            'view-sound',
            'view-video',
            'share-link',
            'daily-visit',
            'admin',
            'view-course',
            'view-page',
            'view-post',
            'view-product',
            'wc_referer',
            'complete-profile',
            'lottery',
            'spin_wheel_score',
            'spin-wheel',
            'spin-wheel',
            'gifts',
            'convert-wallet',
            'import_form_plugins',
            'post-rating',
            'submit_gform',
            'wc_order_cancelled',
            'wc_order',
            'comment',
            'lms_quiz_passed',
            'lms_lesson_complete',
            'lms_course_complete'
        );
    }





}
