<?php namespace Avans;

class db
{
    protected static $log_tbl_score = 'avans_log_score';
    protected static $log_tbl_visit = 'avans_log_visit';
    protected static $log_tbl_score_meta = 'avans_log_meta';
    protected static $log_tbl_gifts_getted = 'avans_log_gifts_getted';
    protected static $log_tbl_refs_logs = 'avans_log_refs';
 
    public function __construct()
    {
        $this->create_tables();
    }

    private function create_tables()
    {
        global $wpdb;
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        if (AVANS_DB_VERSION == get_option('avans_db_version', '1.0.0')) {
            return;
        }

        $charset_collate = $wpdb->get_charset_collate();
        $table_log_score = $wpdb->prefix.self::$log_tbl_score;
        $table_log_visit = $wpdb->prefix.self::$log_tbl_visit;
        $table_log_score_meta = $wpdb->prefix.self::$log_tbl_score_meta;
        $table_log_gifts_getted =  $wpdb->prefix.self::$log_tbl_gifts_getted;
        $table_log_refs =  $wpdb->prefix.self::$log_tbl_refs_logs;

        if ($wpdb->get_var("SHOW TABLES LIKE '{$table_log_score}'") != $table_log_score) {
            $this->create_log_score_table($table_log_score, $charset_collate);
        }

        if ($wpdb->get_var("SHOW TABLES LIKE '{$table_log_visit}'") != $table_log_visit) {
            $this->create_log_visit_table($table_log_visit, $charset_collate);
        }

        if ($wpdb->get_var("SHOW TABLES LIKE '{$table_log_score_meta}'") != $table_log_score_meta) {
            $this->create_log_meta_table($table_log_score_meta, $charset_collate);
        }

        if ($wpdb->get_var("SHOW TABLES LIKE '{$table_log_gifts_getted}'") != $table_log_gifts_getted) {
            $this->create_log_getted_gift_table($table_log_gifts_getted, $charset_collate);
        }

        if ($wpdb->get_var("SHOW TABLES LIKE '{$table_log_refs}'") != $table_log_refs) {
            $this->create_log_refs_table($table_log_refs, $charset_collate);
        }



        update_option('avans_db_version', AVANS_DB_VERSION);
    }


    private function create_log_score_table($table_name, $charset_collate)
    {
        $sql = "CREATE TABLE $table_name (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            user bigint(20) NOT NULL,
            type varchar(50) NOT NULL,
            score int(11) NOT NULL,
            total int(11) NOT NULL,
            status tinyint(1) NOT NULL,
            created_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
            expire_date datetime NULL,
            PRIMARY KEY  (id)
            ) $charset_collate;";
        dbDelta($sql);
    }




    private function create_log_visit_table($table_name, $charset_collate)
    {
        $sql = "CREATE TABLE $table_name (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            ip varchar(30) NOT NULL,
            page_id bigint(20) NOT NULL,
            details longtext NOT NULL,
            created_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY  (id)
            ) $charset_collate;";
        dbDelta($sql);
    }


    private function create_log_meta_table($table_name, $charset_collate)
    {
        $sql = "CREATE TABLE $table_name (
            meta_id bigint(20) NOT NULL AUTO_INCREMENT,
            log_id bigint(20) NOT NULL,
            meta_key varchar(50) NOT NULL,
            meta_value longtext NOT NULL,
            PRIMARY KEY  (meta_id)
            ) $charset_collate;";
        dbDelta($sql);
    }

    private function create_log_getted_gift_table($table_name, $charset_collate)
    {
        $sql = "CREATE TABLE $table_name (
                id bigint(20) NOT NULL AUTO_INCREMENT,
                user bigint(20) NOT NULL,
                gift bigint(20) NOT NULL,
                content longtext NOT NULL,
                status varchar(20) NOT NULL,
                create_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP, 
                PRIMARY KEY  (id)
                ) $charset_collate;";
        dbDelta($sql);
    }

    public function create_log_refs_table($table_name, $charset_collate){
        $sql = "CREATE TABLE $table_name (
        id bigint(20) NOT NULL AUTO_INCREMENT,
        ref_id bigint(20) NOT NULL,
        user_id bigint(20) NOT NULL,    
        type varchar(40) NOT NULL,
        amount bigint(20) NOT NULL,
        order_id bigint(20) NOT NULL,
        status varchar(30) NOT NULL,
        created_at timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP, 
        PRIMARY KEY  (id)
        ) $charset_collate;";
        dbDelta($sql);
    }

}
