<?php namespace Avans\parts;

use Avans\data;
use Avans\func;
use Avans\user;

class completeProfile{


    public static function set_user_info($user_id , $data){
        $settings = AVANS_SETTINGS['panel_settings']['user_complete'];
        $birthday_use = get_user_meta($user_id, AVANS_PREFIX.'birthday_gift_getted', true);
        $is_used_birthdate_gifts = get_user_meta($user_id, AVANS_PREFIX.'birthday_gift_getted', true);

        $send_code_email = false;
        $send_code_mobile = false;

        $args =  ['ID' => $user_id];
        if(func::is_set($data['firstname'])){
            $args['first_name'] = esc_attr($data['firstname']);
        }
        if(func::is_set($data['lastname'])){
            $args['last_name'] = esc_attr($data['lastname']);
        }
        if (func::is_set($data['email']) && !self::is_validation($user_id,'email') ) {
            $args['user_email'] = esc_attr($data['email']);
            $send_code_email = true;
        }

        if(func::is_set($data['firstname']) || func::is_set($data['lastname'])){
            $info = user::get_user_info($user_id);
            $firstname = func::is_set($data['firstname']) ? $data['firstname'] : $info->firstname;
            $lastname = func::is_set($data['lastname']) ? $data['lastname'] : $info->lastname ;
            $args['display_name'] = esc_attr($firstname) .' '. esc_attr($lastname);
        }

        $update_user = wp_update_user($args);

        if (func::is_set($data['mobile']) && !self::is_validation($user_id,'mobile') ) {
            update_user_meta($user_id, 'billing_phone', esc_attr( $data['mobile'] ));
            $send_code_mobile = true;
        }

        if (func::is_set($data['birthdate']) && empty($birthday_use) && !func::is_set($is_used_birthdate_gifts)) {
            $birthdate = func::convert_jalali_to_miladi($data['birthdate']);
            update_user_meta($user_id, AVANS_PREFIX.'birthdate', $birthdate);
        }

        if(is_wp_error($update_user)){
            return false;
        }

    
        $sendEmail = false;
        if((!func::is_set($settings['validation_email']) || $settings['validation_email'] == 'true') && 
            ( func::is_set($data['email']) && !self::is_validation($user_id,'email') )  ): 
            if($send_code_email){
                $sendEmail = self::send_validation_email_code($user_id,$data);
            }
        endif;


        $sendSms = false;
        if( (!func::is_set($settings['validation_mobile']) || $settings['validation_mobile'] == 'true') && 
            ( func::is_set($data['mobile']) && !self::is_validation($user_id,'mobile') ) ): 
            if($send_code_mobile){
                self::send_validation_sms_code($user_id, $data,$settings);
                $sendSms = true;
            }
        endif;



        if( ($send_code_email || $send_code_mobile) && ($sendSms || $sendEmail) ){
            $status = [
                'validate_code' => true,
                'validate_email' => $sendEmail,
                'validate_sms' => $sendSms
            ];
            return $status;
        }


        $result = self::check_for_apply_score($user_id);
        if((int)$result > 0){
            $settings = AVANS_SETTINGS['score_settings']['complete-profile'];
            $unit = func::get_score_label();
            $text = "شما بابت تکمیل و تایید اطلاعات حساب کاربریتان $result $unit گرفتید" ;
            if (!empty($settings['text'])) {
                $text  = str_replace("{score}", $result, $settings['text']);
            }

            return array(
                'completeProfile' => true,
                'message' => $text,
            );
        }
        
        return true;

    }

    public static function check_for_apply_score($user_id){
        $settings = AVANS_SETTINGS['panel_settings']['user_complete'];
        if ($settings['validation_mobile'] != 'true' || $settings['validation_email'] != 'true') {
            $score = 0;
            if (user::is_user_block($user_id)) {
                return $score;
            }
            $validation_mobile = get_user_meta($user_id, AVANS_PREFIX.'validation_mobile', true);
            $validation_email = get_user_meta($user_id, AVANS_PREFIX.'validation_email', true);

            $completeProfileScore = get_user_meta($user_id, AVANS_PREFIX.'complete_profile_score', true);
            if (empty($completeProfileScore) && !($validation_mobile == 'true' && $validation_email == 'true') ) {
                $score = user::add_log($user_id, 'complete-profile');
                if((int)$score > 0){
                    update_user_meta($user_id, AVANS_PREFIX.'complete_profile_score', $score);                }
                return $score;
            }
        }
        return false;
    }

    public static function is_validation($user_id,$type){
        $validation = get_user_meta($user_id, AVANS_PREFIX.'validation_'.$type, true);
        
        if($type == 'mobile'){
            
            $digitsPhone = get_user_meta($user_id, 'digits_phone', true);
            if(!empty($digitsPhone)){
                update_user_meta($user_id, AVANS_PREFIX.'validation_mobile', 'true');
                return true;
            }
            
            $wupp_mobile = get_user_meta($user_id, 'wupp_mobile', true);
            if (!empty($wupp_mobile)) {
                update_user_meta($user_id, AVANS_PREFIX.'validation_mobile', 'true');
                return true;
            }

        }

        if($validation == 'true'){
            return true;
        }

        return false;
    }

    public static function is_send_validation($user_id,$type){
        $validation = get_user_meta($user_id, AVANS_PREFIX.'validation_'.$type, true);
        if (!empty($validation) && $validation != 'true') {
            return true;
        }
        return false;
    }


    public static function resend_verification_code($user_id,$type){
        $status = false;
        $data = user::get_user_info($user_id);
        $settings = AVANS_SETTINGS['panel_settings']['user_complete'];

        $data = [
            'firstname' => $data->firstname,
            'lastname' => $data->lastname,
            'email' => $data->email,
            'mobile' => $data->mobile,
        ];
        if($type == 'email'){
            $status = self::send_validation_email_code($user_id,$data);
        }

        if($type == 'sms'){
            self::send_validation_sms_code($user_id,$data,$settings);
            $status = true;
        }

        return $status;
    }

    public static function send_validation_sms_code($user_id,$data,$settings){
        $verification = self::create_verification_link($user_id, 'mobile');

        $text = data::get_text_sms($settings,$user_id,null,$verification['code']);

        $pattern = '';
        if($settings['sms']['type'] == 'pattern' || !isset($settings['sms']['type'])){
            $pattern = $settings['sms']['pattern'];
        }

        do_action('avans_send_sms', esc_attr($data['mobile']) , $text, $pattern);
    }

    public static function send_validation_email_code($user_id,$data){
        $fullname = esc_attr($data['firstname']).' '.esc_attr($data['lastname']);
        $sitename = get_bloginfo('name');

        $validation_code = self::create_verification_link($user_id, 'email');
        $arguments = [
            'title' => 'تایید حساب ایمیل',
            'email' => $data['email'],
            'site_name' => $sitename,
            'fullname' => $fullname,
            'verification_code' => $validation_code['code'],
            'verification_link' => $validation_code['link'],
        ];

        $send = func::avans_send_email($arguments, AVANS_PATH.'templates/email/verification-email.php');

        return $send;
    }

    public static function create_verification_link($user_id,$type){
        $code = rand(111111,999999);
        $array = [
            'code' => $code,
            'link' => add_query_arg(['user_id'=>$user_id,'code'=>$code],home_url('avans_verification')),
        ];
        $validation = (array)get_user_meta($user_id,AVANS_PREFIX.'validation_'.$type,true);
        array_push($validation,$code);
        update_user_meta($user_id,AVANS_PREFIX.'validation_'.$type,$validation);
        return $array;
    }

    public static function is_valid_user_code_inserted($user_id,$type,$code){
        $validation = get_user_meta($user_id, AVANS_PREFIX.'validation_'.$type, true);

        if( (is_array($validation) && in_array($code,$validation)) || $validation == 'true'){
            update_user_meta($user_id, AVANS_PREFIX.'validation_'.$type, 'true');
            return true;
        }
        return false;
    }

    public static function update_avatar($user_id , $file){
        $result = false;

        if(empty($file['tmp_name'])){
            return $result;
        }

        $uploadPath =  wp_upload_dir()['basedir'];

        $filepath = $uploadPath.'/avans-avatar/';
        if(!file_exists($filepath)){
            wp_mkdir_p($filepath);
        }

        $name_parts = explode('.', $file['name']);
        $filename = $user_id.'.'.end($name_parts);

        if (!file_exists($filepath.$filename)) {
            $result = move_uploaded_file($file['tmp_name'], $filepath.$filename);
        }else{
            unlink($filepath.$filename);
            $result = move_uploaded_file($file['tmp_name'], $filepath.$filename);
        }

        if($result){

            $image = wp_get_image_editor($filepath.$filename);

            if (! is_wp_error($image)) {

                $image->resize(128, 128 , true);
                $resize_image_name = $user_id.'-small.'.end($name_parts);

                $formats = ['jpg','png','jpeg'];
                for($i=0;$i<count($formats);$i++){
                    if (file_exists($filepath.$user_id.'-small.'.$formats[$i])) {
                        unlink($filepath.$user_id.'-small.'.$formats[$i]);
                    }
                }
                
                $save_avatar = $image->save($filepath.$resize_image_name);

                 if (! is_wp_error($save_avatar)) {
                    update_user_meta($user_id, AVANS_PREFIX.'avatar_version', rand(1111,9999));
                    if (file_exists($filepath.$filename)) {
                        unlink($filepath.$filename);
                    }
                 }else{
                    $result = false;
                 }

            }else{
                $result = false;
            }

        }

        return $result;

    }

    public static function get_avans_avatar_url($user_id){
        $upload_path = wp_upload_dir();
        $file_path = $upload_path['basedir'].'/avans-avatar/'.$user_id.'-small.';
        $file_url = $upload_path['baseurl'].'/avans-avatar/'.$user_id.'-small.';   
        $version = get_user_meta($user_id,AVANS_PREFIX.'avatar_version',true);
        $format = ['jpg','jpeg','png','PNG','JPG','JPEG'];
        for($i=0;$i<count($format);$i++){
            if (file_exists($file_path.$format[$i])) {
                return $file_url .$format[$i].'?version='.$version;
            }
        }

        return false;
    }



    public static function apply_score_for_complete_user_info($user_id){
        $score = 0;
        if(user::is_user_block($user_id)){
            return $score;
        }
        $settings = AVANS_SETTINGS['panel_settings']['user_complete'];

        $completeProfileScore = get_user_meta($user_id, AVANS_PREFIX.'complete_profile_score', true);
        
        if($settings['validation_email'] == 'true'){
            $validation_email = get_user_meta($user_id, AVANS_PREFIX.'validation_email', true);
        }else{
            $validation_email = 'true';
        }
        
        if ($settings['validation_mobile'] == 'true') {
            $validation_mobile = get_user_meta($user_id, AVANS_PREFIX.'validation_mobile', true);
        }else{
            $validation_mobile = 'true';
        }


        if($validation_mobile == 'true' && $validation_email == 'true' && empty($completeProfileScore) ){
            $score = user::add_log($user_id,'complete-profile');
            update_user_meta($user_id, AVANS_PREFIX.'complete_profile_score', $score);
        }

        return $score;
    }
}