<?php namespace Avans\parts;

use Avans\data;
use Avans\user;

class gifts
{
    private $user;
    private $options;
    public function __construct($gift_id, $user)
    {
        $this->user = $user;
        $this->options = get_post_meta($gift_id, AVANS_PREFIX.'gift_settings', true);
    }

    public function add_gifts()
    {
        $user_score = user::get_score_valid((int)$this->user);
        
        if ($user_score < (int)$this->options['score_for_get_gift']) {
            return false;
        }

        switch ($this->options['avans_type_gift']) {
            case 'coupon':
                $data = $this->add_gifts_coupon();
            break;
            case 'file':
                $data = $this->add_gifts_downloadable_files();
            break;
            case 'custom':
                $data = $this->add_gifts_custom();
            break;
            case 'product':
                $data = $this->add_gifts_product();
            break;
        }

        return $data;
    }

    private function add_gifts_coupon()
    {
        $data = $this->options;
        $args = array(
            'char' => $data['number_of_charecter_coupen'],
            'expire_date' => $data['expire_date_coupen'],
            'prefix' => $data['prefix_coupen'],
            'suffix' => $data['suffix_coupen'],
            'type' => $data['avans_type_coupon_gift'],
            'amount' => $data['value_percent_coupen'],
            'include_product' => $data['limited_custom_product_for_copon'],
            'include_category' => $data['limited_custom_category_for_copon'],
            'exclude_product' => $data['exclude_custom_product_for_copon'],
            'exclude_category' => $data['exclude_custom_category_for_copon'],
            'min_price' => $data['min_price_for_coupen'],
            'max_price' => $data['max_price_for_coupen'],
            'prefix' => $data['prefix_coupen'],
            'exclude_sale_product' => $data['exclude_sale_product'],
            'max_discount' => $data['max_discount'],
        );
        $coupon = data::create_coupon($args);
        $data = [
            'code' => $coupon,
            'args' => $args,
        ];
        return $data;
    }

    private function add_gifts_downloadable_files()
    {
        $data = [
            'args' => $this->options['download_file_link_for_gift']
        ];
        return $data;
    } 

    private function add_gifts_product()
    {
        $products = $this->options['select_product_for_gift'];
        global $woocommerce;
        $order = wc_create_order(['customer_id' => $this->user]);
        foreach ($products as $product) {
            $order->add_product(wc_get_product(strval($product)), 1);
        }
        $order->update_status("wc-processing", 'Imported order', true);
        update_post_meta($order->get_id(),AVANS_PREFIX.'is_order_prize','true');
        $data = [
            'order_id' => $order->get_id(),
        ];
        return $data;
    }

    private function add_gifts_custom()
    {
        $data = [
            'full_content' => $this->options['custom_content_for_gift'],
        ];
        return $data;
    }
}
