<?php namespace Avans\parts;

use Avans\data;
use Avans\user;

class spin_wheel
{
    private $wheel_id;
    private $user;

    public function __construct($wheel_id, $user)
    {
        $this->wheel_id = $wheel_id;
        $this->user = $user;
    }

    public function get_prize()
    {
        $slices = $this->create_certain_array($this->get_options());
        if (!empty($slices)) {
            shuffle($slices);
            $index = array_rand($slices, 1);
            $this->update_stock($slices[$index]);
            // return ['select'=>$slices[$index],'options'=> $update];
            $result = [
               'index' => $slices[$index],
               'result' => $this->add_gifts($slices[$index]),
            ];
            return $result;
        }
        return false;
    }

    protected function get_options()
    {
        $options = get_post_meta($this->wheel_id, AVANS_PREFIX.'spin_wheel_settings', true);
        return $options['slices'];
    }

    protected function create_certain_array($slices)
    {
        $data = [];
        if (count($slices) > 0) {
            foreach ($slices as $key => $slice) {
                if ( (int)$slice->stock > 0 || trim($slice->stock) == '' ) {
                    for ($j = 0; $j < (int)$slice->possibility; $j++) {
                        array_push($data, $key);
                    }
                }
            }
        }
        return $data;
    }

    protected function update_stock($index)
    {
        $option = get_post_meta($this->wheel_id, AVANS_PREFIX.'spin_wheel_settings', true);
        if (!empty($option['slices'][$index]->stock)) {
            $option['slices'][$index]->stock = $option['slices'][$index]->stock - 1;
            update_post_meta($this->wheel_id, AVANS_PREFIX.'spin_wheel_settings', $option);
        }
        return $this->get_options();
    }

    protected function add_gifts($index)
    {
        $options = $this->get_options()[$index];
        $data = [];
        switch ($options->type) {
            case 'coupon':
                $data = $this->add_gifts_coupon($options);
            break;
            case 'file':
                $data = $this->add_gifts_downloadable_files($options);
            break;
            case 'product':
                $data = $this->add_gifts_product($options);
            break;
            case 'score':
                $data = $this->add_gifts_score($options);
            break;
            case 'custom':
                $data = $this->add_gifts_custom($options);
            break;
            default:
                $data = [];
            break;
        }

        $data['type'] = $options->type;
        return $data;
    }

    private function add_gifts_score($options){

        $expire = [
            'value' => $options->score->score_expire->value,
            'unit' =>  $options->score->score_expire->unit,
        ];

        $details = [
            [
                'key' => 'wheel-id',
                'value' => $this->wheel_id
            ]
        ];
        user::add_log($this->user,'spin_wheel_score',$details, (int)$options->score->score,$expire);

        $data = [
            'args' => [
                'score' => $options->score->score,
                'expier' => $expire,
            ]
        ];
        return $data;

    }

    private function add_gifts_coupon($options)
    {
        $data = $options;
        $args = array(
            'char' => $data->char_coupon,
            'expire_date' => $data->expire_coupon,
            'prefix' => '',
            'suffix' => '',
            'type' => $data->type_coupon,
            'amount' => $data->coupon_value,
            'include_product' => $data->limit_products,
            'include_category' => $data->limit_categorys,
            'exclude_product' => $data->exclude_products,
            'exclude_category' => $data->exclude_categorys,
            'min_price' => $data->min_price,
            'max_price' => $data->max_price,
            'prefix' => $data->prefix,
            'exclude_sale_product' => $data->exclude_sale_product,
            'max_discount' => $data->max_discount,
        );
        $coupon = data::create_coupon($args);
        $data = [
            'code' => $coupon,
            'args' => $args,
        ];
        return $data;
    }

    private function add_gifts_downloadable_files($options)
    {
        $data = [
            'args' => $options->files,
        ];
        return $data;
    }

    private function add_gifts_product($options)
    {
        $products = $options->product;
        global $woocommerce;
        $order = wc_create_order(['customer_id' => $this->user]);
        foreach ($products as $product) {
            $order->add_product(wc_get_product(strval($product)), 1);
        }
        $order->update_status("wc-processing", 'Imported order', true);
        update_post_meta($order->get_id(),AVANS_PREFIX.'is_order_prize','true');
        $data = [
            'order_id' => $order->get_id(),
            'is_shipping' => $options->shipping_info,
        ];
        return $data;
    }

    private function add_gifts_custom($options)
    {
        $data = [
            'full_content' => '',
        ];
        return $data;
    }
}
